#!/usr/bin/env python
# -*- coding: utf-8 -*-
# -*- Python -*-

"""
 @file target_gps_data.py
 @brief This module makes drone target GPS data.
 @date $Date$

 @author 株式会社 東日本計算センター

"""
import sys
import RTC
from logging import getLogger, StreamHandler
from logging import DEBUG as LOG_LV

sys.path.append(".")

# constants
MIN_ALT = 2.0
MAX_ALT = 15.0

CONTROL_MODE_0 = 0
CONTROL_MODE_1 = 1

# Setup logger
logger = getLogger(__name__)
handler = StreamHandler()
handler.setLevel(LOG_LV)
logger.setLevel(LOG_LV)
logger.addHandler(handler)


class TargetGPS(object):
    def __init__(self):
        self._init_variable()

    def _init_variable(self):
        self._mode = CONTROL_MODE_1
        self._AltCoefficient = 1
        self._LatLonCoefficient = 2
        self._LatDiff10m = 0.000137
        self._LonDiff10m = 0.000108
        self._MoveSpeed = 2.0

    def config_param(self, kwargs):
        key1 = key2 = key3 = key4 = key5 = None
        try:
            key1 = kwargs["AltCoefficient"]
            key2 = kwargs["LatLonCoefficient"]
            key3 = kwargs["LatDiff10m"]
            key4 = kwargs["LonDiff10m"]
            key5 = kwargs["MoveSpeed"]
        except KeyError:
            logger.warning("ERROR: KeyError")

        if key1 is not None:
            self._AltCoefficient = key1

        if key2 is not None:
            self._LatLonCoefficient = key2

        if key3 is not None:
            self._LatDiff10m = key3

        if key4 is not None:
            self._LonDiff10m = key4

        if key5 is not None:
            self._MoveSpeed = key5

    def set_controller_mode(self):
        self._mode = CONTROL_MODE_1

    def get_takeoff_landing_data(self, home, takeoff_landing):
        check = lambda x: x > 1 and 1 or x < -1 and -1 or x
        takeoff_landing = check(takeoff_landing)

        target = None
        if takeoff_landing:
            target = RTC.GPSData(RTC.Time(0, 0), RTC.GPSTime(0, 0),
                                 home.latitude, home.longitude, home.altitude,
                                 0.0, 0.0, 0.0, 0.0,
                                 0.0, 0, RTC.GPS_FIX_NORMAL)

            if takeoff_landing == 1:
                target.altitude = 10.0

        return target

    def get_move_target_data(self, drone, front_back, right_left, up_down):
        target = None
        if self._mode == CONTROL_MODE_1:
            target = self._move_mode1(drone, front_back, right_left, up_down)
        else:
            logger.error("Invalid control mode: %d", self._mode)

        return target

    def _move_mode1(self, drone, south_north, east_west, up_down):
        target = RTC.GPSData(RTC.Time(0, 0), RTC.GPSTime(0, 0),
                             0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0,
                             0.0, 0, RTC.GPS_FIX_NORMAL)

        target.latitude = self._move_south_north(drone.latitude, south_north)
        target.longitude = self._move_east_west(drone.longitude, east_west)
        target.altitude = self._move_up_down(drone.altitude, up_down)

        return target

    def _move_south_north(self, drone_lat, operation):
        check = lambda x: x > 1 and 1 or x < -1 and -1 or x
        operation = check(operation)

        diff = ((self._LatDiff10m / 10) *
                self._MoveSpeed *
                self._LatLonCoefficient *
                operation)

        target_lat = drone_lat + diff

        if operation:
            check = lambda x: x > 90 and 90.0 or x < -90 and -90.0 or x
            target_lat = check(target_lat)

        return target_lat

    def _move_east_west(self, drone_lon, operation):
        check = lambda x: x > 1 and 1 or x < -1 and -1 or x
        operation = check(operation)

        diff = ((self._LonDiff10m / 10) *
                self._MoveSpeed *
                self._LatLonCoefficient *
                operation)

        target_lon = drone_lon + diff

        if operation:
            check = lambda x: x > 180 and x-360.0 or x < -180 and x+360 or x
            target_lon = check(target_lon)

        return target_lon

    def _move_up_down(self, drone_alt, operation):
        check = lambda x: x > 1 and 1 or x < -1 and -1 or x
        operation = check(operation)

        diff = self._AltCoefficient * operation
        target_alt = drone_alt + diff

        if operation:
            check = (lambda x: x < MIN_ALT and MIN_ALT or
                     x > MAX_ALT and MAX_ALT or x)
            target_alt = check(target_alt)

        return target_alt


def debug_target_gps_data():
    m = TargetGPS()
    config = {"AltCoefficient": 111,
              "LatLonCoefficient": 222,
              "LatDiff10m": 333.333,
              "LonDiff10m": 444.444,
              "MoveSpeed": 555.5555
              }

    m.config_param(config)
    logger.debug("%d %d %f %f %f",
                 m._AltCoefficient, m._LatLonCoefficient,
                 m._LatDiff10m, m._LonDiff10m, m._MoveSpeed)

    m._init_variable()
    gps_data = RTC.GPSData(RTC.Time(0, 0), RTC.GPSTime(0, 0),
                           37.523763, 139.938025, 3.5, 0.0, 0.0, 0.0, 0.0,
                           0.0, 0, RTC.GPS_FIX_NORMAL)

    logger.debug(m.get_takeoff_landing_data(gps_data, 1))
    logger.debug(m.get_takeoff_landing_data(gps_data, 0))
    logger.debug(m.get_takeoff_landing_data(gps_data, -1))

    logger.debug(m.get_move_target_data(gps_data, 0, 0, 0))
    logger.debug(m.get_move_target_data(gps_data, 1, 1, 1))
    logger.debug(m.get_move_target_data(gps_data, -1, -1, -1))

if __name__ == "__main__":
    debug_target_gps_data()
